#define RunService(x) 

#include <stdio.h>

//CMPE13 Support Library
#include "BOARD.h"

// Microchip libraries
#include <xc.h>
#include <plib.h>



//#define UGLY_WAY

#ifdef UGLY_WAY
//here's the usual way to print an event as it occurs:

typedef enum {
    EVENT_NONE = 0,
    EVENT_MESSAGE_RECEIVED,
    EVENT_MESSAGE_SENT,
    EVENT_NEW_DATA_AVAILABLE,
    EVENT_RESET,
} EventType;

EventType currentEvent;

int main(void)
{
    BOARD_Init();
    printf("Checking for events:\n");

    // <editor-fold defaultstate="collapsed" desc="configure timer"> 
    // Configure Timer 1 using PBCLK as input. This default period will make the LEDs blink at a
    // pretty reasonable rate to start.
    OpenTimer1(T1_ON | T1_SOURCE_INT | T1_PS_1_8, 0xFFFF);

    // Set up the timer interrupt with a priority of 4.
    INTClearFlag(INT_T1);
    INTSetVectorPriority(INT_TIMER_1_VECTOR, INT_PRIORITY_LEVEL_4);
    INTSetVectorSubPriority(INT_TIMER_1_VECTOR, INT_SUB_PRIORITY_LEVEL_0);
    INTEnable(INT_T1, INT_ENABLED);
    // </editor-fold>

    while (1) {
        if (currentEvent != EVENT_NONE) {
            //print out the current event's name:
            switch (currentEvent) {
            case EVENT_MESSAGE_RECEIVED:
                printf("Received EVENT_MESSAGE_RECEIVED\n");
                break;
            case EVENT_MESSAGE_SENT:
                printf("Received EVENT_MESSAGE_SENT\n");
                break;
            case EVENT_NEW_DATA_AVAILABLE:
                printf("Received EVENT_NEW_DATA_AVAILABLE\n");
                break;
            case EVENT_RESET:
                printf("Received EVENT_RESET\n");
                break;
            default: break;
            }

            RunService(currentEvent);
            currentEvent = EVENT_NONE;
        }
    }
}

#endif //UGLY_WAY







//but you can use a stringifying macro to make it a bit easier/shorter!
//#define STRINGIFIED_WAY

#ifdef STRINGIFIED_WAY

typedef enum {
    EVENT_NONE = 0,
    EVENT_MESSAGE_RECEIVED,
    EVENT_MESSAGE_SENT,
    EVENT_NEW_DATA_AVAILABLE,
    EVENT_RESET,
} EventType;

EventType currentEvent;

int main(void)
{
    BOARD_Init();
    printf("Checking for events:\n");

    // <editor-fold defaultstate="collapsed" desc="configure timer"> 
    // Configure Timer 1 using PBCLK as input. This default period will make the LEDs blink at a
    // pretty reasonable rate to start.
    OpenTimer1(T1_ON | T1_SOURCE_INT | T1_PS_1_8, 0xFFFF);

    // Set up the timer interrupt with a priority of 4.
    INTClearFlag(INT_T1);
    INTSetVectorPriority(INT_TIMER_1_VECTOR, INT_PRIORITY_LEVEL_4);
    INTSetVectorSubPriority(INT_TIMER_1_VECTOR, INT_SUB_PRIORITY_LEVEL_0);
    INTEnable(INT_T1, INT_ENABLED);
    // </editor-fold>

#define printcase( x ) case x: printf("Received event " #x "\n"); break; 

    while (1) {
        if (currentEvent != EVENT_NONE) {
            //print out the current event's name:
            switch (currentEvent) {
                printcase(EVENT_MESSAGE_RECEIVED);
                printcase(EVENT_MESSAGE_SENT);
                printcase(EVENT_NEW_DATA_AVAILABLE);
                printcase(EVENT_RESET);
            default: break;
            }

            RunService(currentEvent);
            currentEvent = EVENT_NONE;
        }
    }
}

#endif //STRINGIFIED_WAY








//Or if you want to keep it REALLY neat, how about unrolling one list?
//#define UNROLLED_AND_STRINGIFIED_WAY

#ifdef UNROLLED_AND_STRINGIFIED_WAY

//Use a looped macro to generate both your enums and a matching list of strings:

#define EVENT_LIST(X)                                                         \
    X(EVENT_NONE) , \
    X(EVENT_MESSAGE_RECEIVED) , \
    X(EVENT_MESSAGE_SENT) , \
    X(EVENT_NEW_DATA_AVAILABLE) , \
    X(EVENT_RESET) 

/* declare the enum */
#define EVENTS_ENUM(EVENT) EVENT

typedef enum {
    EVENT_LIST(EVENTS_ENUM)
} EventType;

/* create an array of strings with the enum names... */
#define EVENTS_STRING(EVENT ) #EVENT
const char * const event_string[] = {
    EVENT_LIST(EVENTS_STRING)
};

EventType currentEvent;

int main(void)
{
    BOARD_Init();
    printf("Checking for events:\n");

    // <editor-fold defaultstate="collapsed" desc="configure timer"> 
    // Configure Timer 1 using PBCLK as input. This default period will make the LEDs blink at a
    // pretty reasonable rate to start.
    OpenTimer1(T1_ON | T1_SOURCE_INT | T1_PS_1_8, 0xFFFF);

    // Set up the timer interrupt with a priority of 4.
    INTClearFlag(INT_T1);
    INTSetVectorPriority(INT_TIMER_1_VECTOR, INT_PRIORITY_LEVEL_4);
    INTSetVectorSubPriority(INT_TIMER_1_VECTOR, INT_SUB_PRIORITY_LEVEL_0);
    INTEnable(INT_T1, INT_ENABLED);
    // </editor-fold>


    while (1) {
        if (currentEvent != EVENT_NONE) {
            //print out the current event's name:
            printf("Received event %s\n", event_string[currentEvent]);
            RunService(currentEvent);
            currentEvent = EVENT_NONE;
        }
    }
}

#endif //UNROLLED_AND_STRINGIFIED_WAY







//And if you're using C99+, you can do this 
#define C99_WAY

#ifdef C99_WAY
//here's the usual way to print an event as it occurs:

typedef enum {
    EVENT_NONE = 0,
    EVENT_MESSAGE_RECEIVED,
    EVENT_MESSAGE_SENT,
    EVENT_NEW_DATA_AVAILABLE,
    EVENT_RESET,
} EventType;

const char* event_string[] = {
    [EVENT_NONE] = "EVENT_NONE",
    [EVENT_MESSAGE_RECEIVED] = "EVENT_MESSAGE_RECEIVED",
    [EVENT_MESSAGE_SENT] = "EVENT_MESSAGE_SENT",
    [EVENT_NEW_DATA_AVAILABLE] = "EVENT_NEW_DATA_AVAILABLE",
    [EVENT_RESET] = "EVENT_RESET"
};

//const char* boat_sizes[] = {
//    [SMALL_BOAT_TYPE] = SMALL_BOAT_SIZE,
//};

EventType currentEvent;

int main(void)
{
    BOARD_Init();
    printf("Checking for events:\n");

    // <editor-fold defaultstate="collapsed" desc="configure timer"> 
    // Configure Timer 1 using PBCLK as input. This default period will make the LEDs blink at a
    // pretty reasonable rate to start.
    OpenTimer1(T1_ON | T1_SOURCE_INT | T1_PS_1_8, 0xFFFF);

    // Set up the timer interrupt with a priority of 4.
    INTClearFlag(INT_T1);
    INTSetVectorPriority(INT_TIMER_1_VECTOR, INT_PRIORITY_LEVEL_4);
    INTSetVectorSubPriority(INT_TIMER_1_VECTOR, INT_SUB_PRIORITY_LEVEL_0);
    INTEnable(INT_T1, INT_ENABLED);
    // </editor-fold>

    while (1) {
        if (currentEvent != EVENT_NONE) {

            //print out the current event's name:
            printf("Received event %s\n", event_string[currentEvent]);


            RunService(currentEvent);
            currentEvent = EVENT_NONE;
        }
    }
}

#endif //C99_WAY








/**
 * This timer just throws a random event every second
 */
void __ISR(_TIMER_1_VECTOR, IPL4AUTO) Timer1Handler(void)
{
    static int timer = 0;
    // Clear the interrupt flag.
    INTClearFlag(INT_T1);

    //post an event:
    if (timer++ > 5) {
        currentEvent = rand() % 4 + 1;
        timer = 0;
    }
}